@@ -3,7 +3,7 @@
 
 #' Recode a variable.
 #'
-#' Based on \code{car::\link[car]{recode}}.
+#' Based on \code{\link[car:recode]{car::recode()}}.
 #'
 #' @param var Variable (numeric, character, or factor).
 #' @param recodes Character string: e.g., \code{"lo:1=0; c(2,3)=1; 4=2; 5:hi=3; else=999"}.
@@ -23,7 +23,7 @@ RECODE=function(var, recodes) {
 }
 
 
-#' Rescale likert scales (e.g., from 5-point to 7-point).
+#' Rescale a variable (e.g., from 5-point to 7-point).
 #'
 #' @param var Variable (numeric).
 #' @param from Numeric vector, the range of old scale (e.g., \code{1:5}).
@@ -47,8 +47,8 @@ RESCALE=function(var, from=range(var, na.rm=T), to) {
 
 #' Min-max scaling (min-max normalization).
 #'
-#' This function resembles \code{\link[bruceR]{RESCALE}},
-#' and it is just equivalent to using \code{RESCALE(var, to=0:1)}.
+#' This function resembles \code{\link[bruceR:RESCALE]{RESCALE()}}
+#' and it is just equivalent to \code{RESCALE(var, to=0:1)}.
 #'
 #' @param v Variable (numeric vector).
 #' @param min Minimum value (default is 0).
@@ -69,32 +69,36 @@ scaler=function(v, min=0, max=1) {
 #' Multivariate computation.
 #'
 #' @description
-#' Easily compute the sum, mean, or other indexes of a scale.
-#' Reverse scoring can also be easily implemented, without generating extra variables
-#' (\code{\link{Alpha}} uses a similar method to deal with reverse scoring)!
+#' Easily compute multivariate sum, mean, and other scores.
+#' Reverse scoring can also be easily implemented without saving extra variables.
+#' \code{\link{Alpha}} function uses a similar method to deal with reverse scoring.
 #'
-#' Three ways to specify the variable list:
+#' Three options to specify the variable list:
 #' \enumerate{
 #'   \item \strong{\code{var + items}}: use the common and unique parts of variable names.
-#'   \item \strong{\code{vars}}: manually define the variable list.
-#'   \item \strong{\code{varrange}}: use the start and stop positions.
+#'   \item \strong{\code{vars}}: directly define a variable list.
+#'   \item \strong{\code{varrange}}: use the start and end positions of a variable list.
 #' }
 #'
 #' @param data Data frame.
-#' @param var \strong{[option 1]} Common part across multiple variables (e.g., \code{"RSES", "SWLS"}).
-#' @param items \strong{[option 1]} Unique part across multiple variables (e.g., \code{1:10}).
-#' @param vars \strong{[option 2]} Character vector specifying the variable list (e.g., \code{c("x1", "x2", "x3")}).
-#' @param varrange \strong{[option 3]} Character with \code{":"} specifying the start and stop positions of variables (e.g., \code{"A1:E5"}).
+#' @param var \strong{[option 1]}
+#' Common part across multiple variables (e.g., \code{"RSES", "SWLS"}).
+#' @param items \strong{[option 1]}
+#' Unique part across multiple variables (e.g., \code{1:10}).
+#' @param vars \strong{[option 2]}
+#' Character vector specifying a variable list (e.g., \code{c("E1", "E2", "E3", "E4", "E5")}).
+#' @param varrange \strong{[option 3]}
+#' Character with \code{":"} specifying the start and end positions of a variable list (e.g., \code{"A1:E5"}).
 #' @param value [only for \code{COUNT}] The value to be counted.
 #' @param rev [optional] Reverse-scoring variables. It can be
-#' 1) a numeric vector specifying the positions of reverse-scoring variables (not recommended) or
-#' 2) a character vector directly specifying the variable list (recommended).
-#' @param likert [optional] Range of likert scale (e.g., \code{1:5}, \code{c(1,5)}).
+#' (1) a numeric vector specifying the positions of reverse-scoring variables (not recommended) or
+#' (2) a character vector directly specifying the variable list (recommended).
+#' @param likert [optional] Range of likert scale (e.g., \code{1:5}, \code{c(1, 5)}).
 #' If not provided, it will be automatically estimated from the given data (BUT you should use this carefully).
 #' @param na.rm Ignore missing values. Default is \code{TRUE}.
 #' @param values [only for \code{CONSEC}] Values to be counted as consecutive identical values. Default is all numbers (\code{0:9}).
 #'
-#' @return A vector of computed variable.
+#' @return A vector of computed values.
 #'
 #' @examples
 #' d=data.table(x1=1:5,
@@ -106,15 +110,16 @@ scaler=function(v, min=0, max=1) {
 #' ## I deliberately set this order to show you
 #' ## the difference between "vars" and "varrange".
 #'
-#' d[,":="(na=COUNT(d, "x", 1:5, value=NA),
-#'         n.2=COUNT(d, "x", 1:5, value=2),
-#'         sum=SUM(d, "x", 1:5),
-#'         m1=MEAN(d, "x", 1:5),
-#'         m2=MEAN(d, vars=c("x1", "x4")),
-#'         m3=MEAN(d, varrange="x1:x2", rev="x2", likert=1:5),
-#'         cons1=CONSEC(d, "x", 1:5),
-#'         cons2=CONSEC(d, varrange="x1:x5")
-#'         )]
+#' d[,`:=`(
+#'   na=COUNT(d, "x", 1:5, value=NA),
+#'   n.2=COUNT(d, "x", 1:5, value=2),
+#'   sum=SUM(d, "x", 1:5),
+#'   m1=MEAN(d, "x", 1:5),
+#'   m2=MEAN(d, vars=c("x1", "x4")),
+#'   m3=MEAN(d, varrange="x1:x2", rev="x2", likert=1:5),
+#'   cons1=CONSEC(d, "x", 1:5),
+#'   cons2=CONSEC(d, varrange="x1:x5")
+#' )]
 #' d
 #'
 #' data=as.data.table(bfi)
@@ -136,7 +141,7 @@ convert2vars=function(data,
                       rev=NULL) {
   if(!is.null(varrange)) {
     dn=names(data)
-    varrange=strsplit(varrange, ":")[[1]]
+    varrange=gsub(" ", "", strsplit(varrange, ":")[[1]])
     vars=dn[which(dn==varrange[1]):which(dn==varrange[2])]
   }
   if(is.null(vars)) vars=paste0(var, items)
@@ -164,7 +169,7 @@ COUNT=function(data, var=NULL, items=NULL, vars=NULL, varrange=NULL,
 }
 
 
-#' @describeIn grapes-grapes-COMPUTE-grapes-grapes Compute the \strong{mode} across multiple variables.
+#' @describeIn grapes-grapes-COMPUTE-grapes-grapes Compute \strong{mode} across multiple variables.
 #' @export
 MODE=function(data, var=NULL, items=NULL, vars=NULL, varrange=NULL) {
   getmode=function(v) {
@@ -177,7 +182,7 @@ MODE=function(data, var=NULL, items=NULL, vars=NULL, varrange=NULL) {
 }
 
 
-#' @describeIn grapes-grapes-COMPUTE-grapes-grapes Compute the \strong{sum} across multiple variables.
+#' @describeIn grapes-grapes-COMPUTE-grapes-grapes Compute \strong{sum} across multiple variables.
 #' @export
 SUM=function(data, var=NULL, items=NULL, vars=NULL, varrange=NULL,
              rev=NULL, likert=NULL,
@@ -198,7 +203,7 @@ SUM=function(data, var=NULL, items=NULL, vars=NULL, varrange=NULL,
 }
 
 
-#' @describeIn grapes-grapes-COMPUTE-grapes-grapes Compute the \strong{mean} across multiple variables.
+#' @describeIn grapes-grapes-COMPUTE-grapes-grapes Compute \strong{mean} across multiple variables.
 #' @export
 MEAN=function(data, var=NULL, items=NULL, vars=NULL, varrange=NULL,
               rev=NULL, likert=NULL,
@@ -219,7 +224,7 @@ MEAN=function(data, var=NULL, items=NULL, vars=NULL, varrange=NULL,
 }
 
 
-#' @describeIn grapes-grapes-COMPUTE-grapes-grapes Compute the \strong{standard deviation} across multiple variables.
+#' @describeIn grapes-grapes-COMPUTE-grapes-grapes Compute \strong{standard deviation} across multiple variables.
 #' @importFrom stats sd
 #' @export
 STD=function(data, var=NULL, items=NULL, vars=NULL, varrange=NULL,
@@ -241,7 +246,7 @@ STD=function(data, var=NULL, items=NULL, vars=NULL, varrange=NULL,
 }
 
 
-#' @describeIn grapes-grapes-COMPUTE-grapes-grapes Compute the \strong{consecutive identical digits} across multiple variables (especially useful in detecting careless responding).
+#' @describeIn grapes-grapes-COMPUTE-grapes-grapes Compute \strong{consecutive identical digits} across multiple variables (especially useful in detecting careless responding).
 #' @export
 CONSEC=function(data, var=NULL, items=NULL,
                 vars=NULL,
@@ -264,36 +269,65 @@ CONSEC=function(data, var=NULL, items=NULL,
 #### Reliability, EFA, and CFA ####
 
 
-#' Reliability analysis (Cronbach's \eqn{\alpha} and corrected item-total correlation).
+#' Reliability analysis (Cronbach's \eqn{\alpha} and McDonald's \eqn{\omega}).
 #'
-#' An extension of \code{jmv::\link[jmv]{reliability}}.
+#' @description
+#' An extension of \code{\link[jmv:reliability]{jmv::reliability()}}.
+#' It reports (1) scale reliability statistics
+#' (Cronbach's \eqn{\alpha} and McDonald's \eqn{\omega}) and
+#' (2) item reliability statistics
+#' (item-rest correlation [i.e., corrected item-total correlation]
+#' and what Cronbach's \eqn{\alpha} and McDonald's \eqn{\omega}
+#' would be if the item was dropped).
+#'
+#' Three options to specify the variable list:
+#' \enumerate{
+#'   \item \strong{\code{var + items}}: use the common and unique parts of variable names.
+#'   \item \strong{\code{vars}}: directly define a variable list.
+#'   \item \strong{\code{varrange}}: use the start and end positions of a variable list.
+#' }
 #'
 #' @inheritParams %%COMPUTE%%
 #'
-#' @return No return value.
+#' @return
+#' A result object obtained from \code{\link[jmv:reliability]{jmv::reliability()}}.
 #'
 #' @examples
-#' ?psych::bfi
-#' Alpha(bfi, "E", 1:5)  # "E1" & "E2" should be reverse scored; see ?bfi
+#' # see ?psych::bfi
+#' Alpha(bfi, "E", 1:5)  # "E1" & "E2" should be reverse scored
 #' Alpha(bfi, "E", 1:5, rev=1:2)  # correct
 #' Alpha(bfi, "E", 1:5, rev=c("E1", "E2"))  # also correct
+#' Alpha(bfi, vars=c("E1", "E2", "E3", "E4", "E5"), rev=c("E1", "E2"))
+#' Alpha(bfi, varrange="E1:E5", rev=c("E1", "E2"))
+#'
+#' # using dplyr::select()
+#' bfi %>% select(E1, E2, E3, E4, E5) %>%
+#'   Alpha(vars=names(.), rev=c("E1", "E2"))
 #'
 #' @seealso
-#' \code{jmv::\link[jmv]{reliability}}
+#' \code{\link{MEAN}}
 #'
 #' @export
-Alpha=function(data, var, items, vars=NULL, rev=NULL) {
+Alpha=function(data, var, items, vars=NULL, varrange=NULL, rev=NULL) {
+  if(!is.null(varrange)) {
+    dn=names(data)
+    varrange=gsub(" ", "", strsplit(varrange, ":")[[1]])
+    vars=dn[which(dn==varrange[1]):which(dn==varrange[2])]
+  }
   if(is.null(vars)) vars=paste0(var, items)
   if(is.numeric(rev)) rev=paste0(var, rev)
-  jmv::reliability(data, vars=eval(vars), revItems=eval(rev),
-                   meanScale=TRUE, sdScale=TRUE,
-                   itemRestCor=TRUE, alphaItems=TRUE)
+  rel=jmv::reliability(data, vars=eval(vars), revItems=eval(rev),
+                       meanScale=TRUE, sdScale=TRUE,
+                       alphaScale=TRUE, omegaScale=TRUE,
+                       itemRestCor=TRUE, alphaItems=TRUE, omegaItems=TRUE)
+  rel$items$setTitle("Item Reliability Statistics (if item is dropped)")
+  return(rel)
 }
 
 
 #' Exploratory factor analysis (EFA).
 #'
-#' An extension of \code{jmv::\link[jmv]{efa}}.
+#' An extension of \code{\link[jmv:efa]{jmv::efa()}}.
 #'
 #' @inheritParams %%COMPUTE%%
 #'
@@ -311,7 +345,7 @@ Alpha=function(data, var, items, vars=NULL, rev=NULL) {
 #' @note It does not have the extraction method "Principal Components". You may still use SPSS.
 #'
 #' @seealso
-#' \code{jmv::\link[jmv]{efa}}
+#' \code{\link[jmv:efa]{jmv::efa()}}
 #'
 #' @examples
 #' \donttest{EFA(bfi, "E[1:5] + A[1:5] + C[1:5] + N[1:5] + O[1:5]", method="fixed", nFactors=5)
@@ -388,24 +422,26 @@ modelCFA.trans=function(style=c("jmv", "lavaan"),
 
 #' Confirmatory factor analysis (CFA).
 #'
-#' An extension of \code{jmv::\link[jmv]{cfa}} and \code{lavaan::\link[lavaan]{cfa}}.
+#' An extension of \code{\link[jmv:cfa]{jmv::cfa()}} and \code{\link[lavaan:cfa]{lavaan::cfa()}}.
 #'
 #' @inheritParams %%COMPUTE%%
 #' @param model Model formula. See examples.
 #' @param highorder High-order factor. Default is \code{""}.
 #' @param orthogonal Default is \code{FALSE}. If \code{TRUE}, all covariances among latent variables are set to zero, and only "lavaan" style will be output.
 #' @param missing Default is \code{"listwise"}. Alternative is \code{"fiml"} (using "Full Information Maximum Likelihood" method to estimate the model).
-#' @param style \code{"jmv"}, \code{"lavaan"}, or both (default).
+#' @param style \code{"jmv"}, \code{"lavaan"} (default), or both \code{c("jmv", "lavaan")}.
 #' If the model has high-order factors, only "lavaan" style will be output.
 #' @param CI \code{TRUE} or \code{FALSE} (default), provide confidence intervals for the model estimates.
 #' @param MI \code{TRUE} or \code{FALSE} (default), provide modification indices for the parameters not included in the model.
 #'
-#' @return A list of results returned by \code{jmv::cfa} and \code{lavaan::cfa}.
+#' @return
+#' A list of results returned by \code{\link[jmv:cfa]{jmv::cfa()}}
+#' and \code{\link[lavaan:cfa]{lavaan::cfa()}}.
 #'
 #' @seealso
-#' \code{jmv::\link[jmv]{cfa}}
+#' \code{\link[jmv:cfa]{jmv::cfa()}}
 #'
-#' \code{lavaan::\link[lavaan]{cfa}}
+#' \code{\link[lavaan:cfa]{lavaan::cfa()}}
 #'
 #' @examples
 #' \donttest{data.cfa=lavaan::HolzingerSwineford1939
@@ -422,7 +458,7 @@ modelCFA.trans=function(style=c("jmv", "lavaan"),
 #' @export
 CFA=function(data, model="A =~ a[1:5]; B =~ b[c(1,3,5)]; C =~ c1 + c2 + c3",
              highorder="", orthogonal=FALSE, missing="listwise",
-             style=c("jmv", "lavaan"), CI=FALSE, MI=FALSE) {
+             style="lavaan", CI=FALSE, MI=FALSE) {
   model.jmv=modelCFA.trans("jmv", model)
   model.lav=modelCFA.trans("lavaan", model, highorder)
   if(orthogonal==TRUE | highorder!="") style="lavaan"
@@ -443,8 +479,7 @@ CFA=function(data, model="A =~ a[1:5]; B =~ b[c(1,3,5)]; C =~ c1 + c2 + c3",
                      # pathDiagram=plot,
                      fitMeasures=c("cfi", "tli", "rmsea", "srmr", "aic", "bic"),
                      miss=missing) # fiml (default), listwise
-    # cat(r$modelSyntax)
-    cat("#### jamovi style output ####\n")
+    cat("\n#### jamovi style output ####\n")
     print(fit.jmv)
     results=c(results, fit.jmv=fit.jmv)
   }
@@ -457,8 +492,8 @@ CFA=function(data, model="A =~ a[1:5]; B =~ b[c(1,3,5)]; C =~ c1 + c2 + c3",
                         # FALSE: fixing the factor loading of the first indicator to 1
                         orthogonal=orthogonal,
                         missing=missing) # fiml, listwise (default)
-    cat("#### lavaan style output ####\n\n")
-    summary(fit.lav, fit.measures=TRUE, standard=TRUE)
+    cat("\n#### lavaan style output ####\n\n")
+    lavaan::summary(fit.lav, fit.measures=TRUE, standard=TRUE)
     if(MI) print(lavaan::modificationIndices(fit.lav))
     # if(plot) semPlot::semPaths(fit.lav, "std", curveAdjacent=TRUE,
     #                            style="lisrel", nDigits=2, edge.label.cex=1)
